#
# Copyright (c) 2008-2019 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

from persistent import Persistent
from zope.interface import implementer
from zope.location import Location
from zope.schema import Choice
from zope.schema.fieldproperty import FieldProperty
from zope.schema.interfaces import IChoice
from zope.schema.vocabulary import SimpleTerm, SimpleVocabulary

from onf_website.shared.contact import IContactManager
from onf_website.shared.contact.interfaces.field import CONTACT_THEME_FIELD_INFO_ANNOTATION_KEY, \
    CONTACT_THEME_FIELD_VALUES_VOCABULARY, IContactThemeFieldInfo
from onf_website.shared.contact.interfaces.theme import IContactThemeManager
from pyams_content.shared.form.field import ChoiceFieldFactory
from pyams_content.shared.form.interfaces import IFormField, IFormFieldFactory
from pyams_i18n.interfaces import II18n
from pyams_utils.adapter import adapter_config, get_annotation_adapter
from pyams_utils.factory import factory_config
from pyams_utils.registry import query_utility, utility_config
from pyams_utils.request import query_request
from pyams_utils.vocabulary import vocabulary_config


from onf_website import _


class IContactThemeField(IChoice):
    """Contact theme field interface"""


@implementer(IContactThemeField)
class ContactThemeField(Choice):
    """Contact theme field"""


@utility_config(name='contact_theme', provides=IFormFieldFactory)
class ContactThemeFormFieldFactory(ChoiceFieldFactory):
    """Contact theme form field factory"""

    label = _("Contact theme")
    weight = 101

    field_factory = ContactThemeField

    def get_schema_field(self, field):
        i18n = II18n(field)
        result = self.field_factory(title=i18n.query_attribute('label'),
                                    description=i18n.query_attribute('description'),
                                    required=field.required,
                                    default=i18n.query_attribute('default'),
                                    vocabulary=ContactThemeFieldValuesVocabulary(field))
        result.__name__ = field.name
        return result


@factory_config(IContactThemeFieldInfo)
class ContactThemeFieldInfo(Persistent, Location):
    """Contact themes field information persistent class"""

    themes = FieldProperty(IContactThemeFieldInfo['themes'])


@adapter_config(context=IFormField, provides=IContactThemeFieldInfo)
def contact_theme_form_field_info_factory(field):
    """Contact theme form field information factory"""
    if field.field_type != 'contact_theme':
        return None
    return get_annotation_adapter(field, CONTACT_THEME_FIELD_INFO_ANNOTATION_KEY,
                                  IContactThemeFieldInfo, name='++info++')


@vocabulary_config(name=CONTACT_THEME_FIELD_VALUES_VOCABULARY)
class ContactThemeFieldValuesVocabulary(SimpleVocabulary):
    """Contact themes field values vocabulary"""

    def __init__(self, context=None):
        terms = []
        field_info = IContactThemeFieldInfo(context, None)
        if field_info is not None:
            contacts = query_utility(IContactManager)
            if contacts is not None:
                manager = IContactThemeManager(contacts)
                request = query_request()
                terms.append(SimpleTerm('', title=request.localizer.translate(_("Please select a theme..."))))
                for name in IContactThemeFieldInfo(context).themes:
                    theme = manager.get(name)
                    terms.append(SimpleTerm(name, title=theme.get_label(request)))
        super(ContactThemeFieldValuesVocabulary, self).__init__(terms)
