#
# Copyright (c) 2008-2019 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

from persistent import Persistent
from pyramid_mailer import IMailer
from zope.schema.fieldproperty import FieldProperty

from onf_website.shared.contact import IContactManager
from onf_website.shared.contact.interfaces.handler import IContactFormHandlerInfo, IContactFormHandlerTarget, \
    CONTACT_FORM_HANDLER_ANNOTATIONS_KEY
from pyams_content.shared.form import IFormHandler, IFormFieldContainer
from pyams_i18n.interfaces import II18n
from pyams_mail.message import HTMLMessage
from pyams_security.interfaces import ISecurityManager
from pyams_security.interfaces.notification import INotificationSettings
from pyams_utils.adapter import adapter_config, get_annotation_adapter
from pyams_utils.factory import factory_config
from pyams_utils.list import boolean_iter
from pyams_utils.registry import utility_config, query_utility, get_utility
from pyams_utils.request import query_request


__docformat__ = 'restructuredtext'

from onf_website import _


@factory_config(IContactFormHandlerInfo)
class ContactFormHandlerInfo(Persistent):
    """Contact form handler persistent info"""

    crm_source = FieldProperty(IContactFormHandlerInfo['crm_source'])
    source_address = FieldProperty(IContactFormHandlerInfo['source_address'])
    source_name = FieldProperty(IContactFormHandlerInfo['source_name'])
    target_address = FieldProperty(IContactFormHandlerInfo['target_address'])
    target_name = FieldProperty(IContactFormHandlerInfo['target_name'])


@adapter_config(context=IContactFormHandlerTarget, provides=IContactFormHandlerInfo)
def contact_form_handler_factory(context):
    """Contact form handler information factory"""
    return get_annotation_adapter(context, CONTACT_FORM_HANDLER_ANNOTATIONS_KEY, IContactFormHandlerInfo)


@utility_config(name='contact', provides=IFormHandler)
class ContactFormHandler(object):
    """Contact form handler"""

    label = _("Contact form handler")
    target_interface = IContactFormHandlerTarget
    handler_info = IContactFormHandlerInfo

    def handle(self, form, data):
        contacts = query_utility(IContactManager)
        if contacts is None:
            return
        # check theme
        fields = IFormFieldContainer(form)
        has_themes, themes_field = boolean_iter(fields.find_fields('contact_theme'))
        if not has_themes:
            return
        theme = data.get(next(themes_field).name)
        # check postal code
        postal_code = None
        has_code, postal_code_field = boolean_iter(fields.find_fields('postal_code'))
        if has_code:
            postal_code = data.get(next(postal_code_field).name)
        # extract contacts list
        has_items, items = boolean_iter(contacts.get_contacts_matching(theme, postal_code))
        if has_items:
            security = get_utility(ISecurityManager)
            settings = INotificationSettings(security)
            if settings.enable_notifications:
                mailer = query_utility(IMailer, name=settings.mailer)
                if mailer is not None:
                    i18n = II18n(form)
                    request = query_request()
                    handler_info = self.handler_info(form)
                    for contact, assignment in items:
                        name = contact.contact_name
                        mail = (assignment.mail_address if assignment is not None else None) or contact.mail_address
                        body = '<br />'.join(('{}: {}'.format(k, v) for k, v in data.items()))
                        message = HTMLMessage(
                            subject='[{}] {}'.format(settings.subject_prefix,
                                                     i18n.query_attribute('user_title', request=request)),
                            fromaddr='{} <{}>'.format(handler_info.source_name, handler_info.source_address),
                            toaddr='{} <{}>'.format(name, mail),
                            html=body)
                        mailer.send(message)
